@@ -6,6 +6,8 @@ import copy
 import NotionDump
 from NotionDump.utils import content_format, common_op
 from NotionDump.utils import internal_var
+from urllib.parse import unquote
+from NotionDump.utils.content_format import color_transformer, color_transformer_db, format_date_or_time
 
 
 class BaseParser:
@@ -30,7 +32,11 @@ class BaseParser:
     def __annotations_parser(block_handle, str_plain):
         if str_plain is None or str_plain == "":
             return ""
-        str_ret = str_plain
+        last_char = str_plain[-1:]
+        if last_char == "\n" or last_char == "\t":
+            str_ret = str_plain[0:-1]
+        else:
+            str_ret = str_plain
         if block_handle["code"]:
             str_ret = "`" + str_ret + "`"
         if block_handle["underline"]:
@@ -40,56 +46,66 @@ class BaseParser:
         if block_handle["italic"]:
             str_ret = "*" + str_ret + "*"
         if block_handle["color"] != "default":
-            # 添加颜色
-            str_ret = "<font color=" + block_handle["color"] + ">" + str_ret + "</font>"
+            # 添加颜色，区分背景色和前景色
+            if NotionDump.S_THEME_TYPE == "markdown":
+                # 使用markdown默认的高亮来渲染所有的颜色类型
+                str_ret = NotionDump.MD_HIGHLIGHT + str_ret + NotionDump.MD_HIGHLIGHT
+            else:
+                if block_handle["color"].find("_background") != -1:
+                    bg_color = block_handle["color"][0:block_handle["color"].rfind('_')]
+                    str_ret = "<span style=\"background-color:" + color_transformer(bg_color, background=True) + "\">" + str_ret + "</span>"
+                else:
+                    str_ret = "<font color=\"" + color_transformer(block_handle["color"], background=False) + "\">" + str_ret + "</font>"
         if block_handle["strikethrough"]:
             str_ret = "~~" + str_ret + "~~"
+        if last_char == "\n" or last_char == "\t":
+            str_ret += last_char
         return str_ret
 
-    def __text_parser(self, block_handle, parser_type=NotionDump.PARSER_TYPE_PLAIN):
+    def __text_parser(self, block_handle, parser_type=NotionDump.PARSER_TYPE_PLAIN, is_db_title=False):
         if block_handle["type"] != "text":
             common_op.debug_log(
                 "text type error! id=" + self.base_id + " not type " + block_handle["type"],
                 level=NotionDump.DUMP_MODE_DEFAULT)
             return ""
-        text_str = block_handle["plain_text"]
+
+        text_str = ""
+        if "plain_text" in block_handle:
+            text_str = block_handle["plain_text"]
         if text_str is None:
             text_str = ""
         # 如果有链接
         text_url = block_handle["href"]
-        if text_url is not None and parser_type == NotionDump.PARSER_TYPE_MD:
+        if text_url is not None and parser_type == NotionDump.PARSER_TYPE_MD and not is_db_title:  # 数据库标题越过链接解析
             # 文字有链接内容，分为网络链接和本地链接
-            if text_url.startswith("http"):
+            if text_url.startswith("http") or not text_url.startswith("/"):
                 # 网络链接，直接一步到位
                 text_str = content_format.get_url_format(text_url, text_str)
             else:
-                # Page类型，等待重定位
+                # Page或者数据库类型，等待重定位
                 if text_url.find("=") != -1:
-                    page_id = text_url[text_url.rfind("/") + 1:text_url.rfind("?")]
-                    common_op.debug_log("### page id " + page_id + " is database")
-                    common_op.add_new_child_page(
-                        self.child_pages,
-                        key_id=page_id + "_" + text_str,
-                        link_id=page_id,
-                        page_type="database",
-                        page_name=text_str
-                    )
+                    id_type = "database"
+                    page_id = text_url[text_url.rfind("/") + 1:text_url.rfind("?")].replace('-', '')
                 else:
-                    page_id = text_url[text_url.rfind("/") + 1:]
-                    common_op.debug_log("### page id " + page_id + " is page")
+                    id_type = "page"
+                    page_id = text_url[text_url.rfind("/") + 1:].replace('-', '')
+                if len(page_id) == NotionDump.ID_LEN:
+                    common_op.debug_log("### page id " + page_id + " is " + id_type)
                     common_op.add_new_child_page(
                         self.child_pages,
                         key_id=page_id + "_" + text_str,
                         link_id=page_id,
-                        page_type="page",
+                        link_src=text_url,
+                        page_type=id_type,
                         page_name=text_str
                     )
-
-                # 将页面保存，等待进一步递归操作
-                # 保存子页面信息
-                common_op.debug_log("child_page_parser add page id = " + page_id + "_" + text_str, level=NotionDump.DUMP_MODE_DEFAULT)
-                text_str = content_format.get_page_format_md(page_id + "_" + text_str, text_str,
-                                                             export_child=self.export_child)
+                    # 将页面保存，等待进一步递归操作
+                    # 保存子页面信息
+                    common_op.debug_log("child_page_parser add page id = " + page_id + "_" + text_str, level=NotionDump.DUMP_MODE_DEFAULT)
+                    text_str = content_format.get_page_format_md(page_id + "_" + text_str, text_str,
+                                                                 export_child=self.export_child)
+                else:
+                    text_str = content_format.get_url_format("", text_str)
 
         if parser_type == NotionDump.PARSER_TYPE_MD:
             # 解析annotations部分，为text_str添加格式
@@ -97,26 +113,30 @@ class BaseParser:
         else:
             return text_str
 
-    def __text_block_parser(self, block_handle, parser_type=NotionDump.PARSER_TYPE_PLAIN):
+    def __text_block_parser(self, block_handle, parser_type=NotionDump.PARSER_TYPE_PLAIN, is_db_title=False):
         paragraph_ret = ""
         if block_handle["type"] == "text":
             paragraph_ret = self.__text_parser(block_handle, parser_type)
         elif block_handle["type"] == "equation":
             paragraph_ret = self.__equation_inline_parser(block_handle)
         elif block_handle["type"] == "mention":
-            paragraph_ret = self.__mention_parser(block_handle, parser_type)
+            paragraph_ret = self.__mention_parser(block_handle, parser_type, is_db_title=is_db_title)
         else:
             common_op.debug_log(
                 "text type " + block_handle["type"] + " error! parent_id= " + self.base_id,
                 level=NotionDump.DUMP_MODE_DEFAULT)
         return paragraph_ret
 
-    def __text_list_parser(self, text_list, parser_type=NotionDump.PARSER_TYPE_PLAIN):
+    def __text_list_parser(self, text_list, parser_type=NotionDump.PARSER_TYPE_PLAIN, is_db=False, is_db_title=False):
         plain_text = ""
         if text_list is not None:
             for text_block in text_list:
-                plain_text += self.__text_block_parser(text_block, parser_type)
-        return plain_text
+                plain_text += self.__text_block_parser(text_block, parser_type, is_db_title=is_db_title)
+        if is_db:
+            # 数据库内容特殊字符校对
+            return plain_text.replace("|", "\\|")
+        else:
+            return plain_text
 
     # TODO : people只获取了名字和ID，后续可以做深度解析用户相关内容
     def __people_parser(self, block_handle):
@@ -154,12 +174,12 @@ class BaseParser:
         common_op.add_new_child_page(
             self.child_pages,
             key_id=file_id,
-            link_id=file_url,
+            link_src=file_url,
             page_type="file",
             page_name=filename
         )
         common_op.debug_log(
-            "file_parser add page id = " + file_id + "name : " + filename, level=NotionDump.DUMP_MODE_DEFAULT)
+            "file_parser add page id = " + file_id + " name : " + filename, level=NotionDump.DUMP_MODE_DEFAULT)
         common_op.debug_log(internal_var.PAGE_DIC)
         common_op.debug_log("#############")
         common_op.debug_log(self.child_pages)
@@ -199,7 +219,7 @@ class BaseParser:
         return page_body["id"].replace('-', '')
 
     # 提及到其它页面，日期，用户
-    def __mention_parser(self, block_handle, parser_type=NotionDump.PARSER_TYPE_PLAIN):
+    def __mention_parser(self, block_handle, parser_type=NotionDump.PARSER_TYPE_PLAIN, is_db_title=False):
         if block_handle["type"] != "mention":
             common_op.debug_log("mention type error! parent_id= " + self.base_id, level=NotionDump.DUMP_MODE_DEFAULT)
             return ""
@@ -218,48 +238,58 @@ class BaseParser:
             common_op.debug_log("__mention_parser add database id = " + database_id)
             # 获取页面的名字
             database_name = block_handle["plain_text"]
-            common_op.add_new_child_page(
-                self.child_pages,
-                key_id=key_id,
-                link_id=database_id,
-                page_type="database",
-                page_name=database_name
-            )
-            common_op.debug_log(
-                "file_parser add page id = " + key_id + "name : " + database_name, level=NotionDump.DUMP_MODE_DEFAULT)
-            common_op.debug_log(internal_var.PAGE_DIC)
-            common_op.debug_log("#############")
-            common_op.debug_log(self.child_pages)
-
-            if parser_type == NotionDump.PARSER_TYPE_MD:
-                mention_plain = content_format.get_page_format_md(key_id, database_name, export_child=self.export_child)
-            else:
+            database_link = block_handle["href"]
+            if is_db_title:
                 mention_plain = database_name
+            else:
+                common_op.add_new_child_page(
+                    self.child_pages,
+                    key_id=key_id,
+                    link_id=database_id,
+                    link_src=database_link,
+                    page_type="database",
+                    page_name=database_name
+                )
+                common_op.debug_log(
+                    "file_parser add page id = " + key_id + " name : " + database_name, level=NotionDump.DUMP_MODE_DEFAULT)
+                common_op.debug_log(internal_var.PAGE_DIC)
+                common_op.debug_log("#############")
+                common_op.debug_log(self.child_pages)
+
+                if parser_type == NotionDump.PARSER_TYPE_MD:
+                    mention_plain = content_format.get_page_format_md(key_id, database_name, export_child=self.export_child)
+                else:
+                    mention_plain = database_name
         elif mention_body["type"] == "page":
             page_id = self.__page_parser(mention_body)
             key_id = page_id + "_mention"
             common_op.debug_log("__mention_parser add page id = " + page_id)
             # 获取页面的名字
             page_name = block_handle["plain_text"]
+            page_link = block_handle["href"]
 
-            # 提及页面按照链接页面处理
-            common_op.add_new_child_page(
-                self.child_pages,
-                key_id=key_id,
-                link_id=page_id,
-                page_type="page",
-                page_name=page_name
-            )
-            common_op.debug_log(
-                "file_parser add page id = " + key_id + "name : " + page_name, level=NotionDump.DUMP_MODE_DEFAULT)
-            common_op.debug_log(internal_var.PAGE_DIC)
-            common_op.debug_log("#############")
-            common_op.debug_log(self.child_pages)
-
-            if parser_type == NotionDump.PARSER_TYPE_MD:
-                mention_plain = content_format.get_page_format_md(key_id, page_name, export_child=self.export_child)
-            else:
+            if is_db_title:
                 mention_plain = page_name
+            else:
+                # 提及页面按照链接页面处理
+                common_op.add_new_child_page(
+                    self.child_pages,
+                    key_id=key_id,
+                    link_id=page_id,
+                    link_src=page_link,
+                    page_type="page",
+                    page_name=page_name
+                )
+                common_op.debug_log(
+                    "file_parser add page id = " + key_id + " name : " + page_name, level=NotionDump.DUMP_MODE_DEFAULT)
+                common_op.debug_log(internal_var.PAGE_DIC)
+                common_op.debug_log("#############")
+                common_op.debug_log(self.child_pages)
+
+                if parser_type == NotionDump.PARSER_TYPE_MD:
+                    mention_plain = content_format.get_page_format_md(key_id, page_name, export_child=self.export_child)
+                else:
+                    mention_plain = page_name
         else:
             common_op.debug_log("unknown mention type " + mention_body["type"], level=NotionDump.DUMP_MODE_DEFAULT)
 
@@ -286,17 +316,24 @@ class BaseParser:
             common_op.debug_log("title type error! parent_id= " + self.base_id + " id= " + block_handle["id"],
                                 level=NotionDump.DUMP_MODE_DEFAULT)
             return ""
-        title_ret = self.__text_list_parser(block_handle["title"], parser_type)
-        if title_ret != "":
+        db_page_title = self.__text_list_parser(block_handle["title"], parser_type, is_db=True, is_db_title=True)
+        if page_id == "":
+            # 如果page id是空的，说明只想要内容，不需要重定位
+            return db_page_title
+
+        if db_page_title != "":
             # 如果存在子Page就加入到待解析队列
-            common_op.debug_log("title ret = " + title_ret)
-            common_op.debug_log("title_parser add page id = " + page_id, level=NotionDump.DUMP_MODE_DEFAULT)
+            common_op.debug_log("title ret = " + db_page_title)
+            if parser_type != NotionDump.PARSER_TYPE_PLAIN:
+                common_op.debug_log("title_parser add page id = " + page_id, level=NotionDump.DUMP_MODE_DEFAULT)
+            else:
+                common_op.debug_log("title_parser add page id = " + page_id)
             # 数据库里的都是子页面
-            common_op.add_new_child_page(self.child_pages, key_id=page_id, page_name=title_ret)
+            common_op.add_new_child_page(self.child_pages, key_id=page_id, page_name=db_page_title)
 
             # 如果有子页面就添加一个占位符，之后方便重定位
-            title_ret = content_format.get_database_title_format(page_id, title_ret, self.export_child)
-        return title_ret
+            db_page_title = content_format.get_database_title_format(page_id, db_page_title, self.export_child)
+        return db_page_title
 
     # 数据库 rich_text
     def rich_text_parser(self, block_handle, parser_type=NotionDump.PARSER_TYPE_PLAIN):
@@ -304,10 +341,10 @@ class BaseParser:
             common_op.debug_log("rich_text type error! parent_id= " + self.base_id + " id= " + block_handle["id"],
                                 level=NotionDump.DUMP_MODE_DEFAULT)
             return ""
-        return self.__text_list_parser(block_handle["rich_text"], parser_type)
+        return self.__text_list_parser(block_handle["rich_text"], parser_type, is_db=True)
 
     # 数据库 multi_select
-    def multi_select_parser(self, block_handle):
+    def multi_select_parser(self, block_handle, parser_type=NotionDump.PARSER_TYPE_PLAIN):
         if block_handle["type"] != "multi_select":
             common_op.debug_log("multi_select type error! parent_id= " + self.base_id + " id= " + block_handle["id"],
                                 level=NotionDump.DUMP_MODE_DEFAULT)
@@ -319,11 +356,16 @@ class BaseParser:
         for multi_select in multi_select_list:
             if ret_str != "":
                 ret_str += ","  # 多个选项之间用“,”分割
-            ret_str += multi_select["name"]
+            if parser_type == NotionDump.PARSER_TYPE_MD:
+                ret_str += "<span style=\"background-color:" \
+                           + color_transformer_db(multi_select["color"]) \
+                           + "\">&nbsp;" + multi_select["name"] + "&nbsp;</span>"
+            else:
+                ret_str += multi_select["name"]
         return ret_str
 
     # 数据库 select
-    def select_parser(self, block_handle):
+    def select_parser(self, block_handle, parser_type=NotionDump.PARSER_TYPE_PLAIN):
         if block_handle["type"] != "select":
             common_op.debug_log("select type error! parent_id= " + self.base_id + " id= " + block_handle["id"],
                                 level=NotionDump.DUMP_MODE_DEFAULT)
@@ -332,7 +374,12 @@ class BaseParser:
         ret_str = ""
         if select is None:
             return ret_str
-        ret_str = select["name"]
+        if parser_type == NotionDump.PARSER_TYPE_MD:
+            ret_str = "<span style=\"background-color:" \
+                + color_transformer_db(select["color"]) \
+                + "\">&nbsp;" + select["name"] + "&nbsp;</span>"
+        else:
+            ret_str = select["name"]
         return ret_str
 
     # 数据库 url
@@ -366,9 +413,9 @@ class BaseParser:
             return ""
         checkbox = block_handle["checkbox"]
         if checkbox is True:
-            ret_str = "true"
+            ret_str = NotionDump.MD_BOOL_TRUE
         else:
-            ret_str = "false"
+            ret_str = NotionDump.MD_BOOL_FALSE
         return ret_str
 
     # 数据库 phone_number
@@ -421,7 +468,7 @@ class BaseParser:
         if number is None:
             return ret_str
         ret_str = number
-        return ret_str
+        return str(ret_str)
 
     # 数据库 files
     def files_parser(self, block_handle, parser_type=NotionDump.PARSER_TYPE_PLAIN):
@@ -442,17 +489,51 @@ class BaseParser:
             ret_str += self.__db_file_parser(file, parser_type)
         return ret_str
 
-    # 数据库 rollup 数据
-    def rollup_parser(self, block_handle):
-        if block_handle["type"] != "rollup":
-            common_op.debug_log("rollup type error! parent_id= " + self.base_id + " id= " + block_handle["id"],
+    # 数据库 relation 数据
+    def relation_parser(self, block_handle):
+        if block_handle["type"] != "relation":
+            common_op.debug_log("relation type error! parent_id= " + self.base_id + " id= " + block_handle["id"],
                                 level=NotionDump.DUMP_MODE_DEFAULT)
             return ""
-        rollup_block = block_handle["rollup"]
+        relation_list = block_handle["relation"]
+        relation_ret = ""
+        for relation_item in relation_list:
+            relation_id = relation_item["id"].replace("-", "")
+            # 按照软连接处理
+            common_op.add_new_child_page(
+                self.child_pages,
+                key_id=relation_id + "_relation",
+                link_id=relation_id,
+                page_type="page",
+                page_name=""
+            )
+            if relation_ret != "":
+                relation_ret += ","
+            relation_ret += content_format.get_database_title_format(relation_id + "_relation", "", self.export_child)
+        return relation_ret
+
+    # 数据库 formula 数据
+    def formula_parser(self, block_handle):
+        if block_handle["type"] != "formula":
+            common_op.debug_log("formula type error! parent_id= " + self.base_id + " id= " + block_handle["id"],
+                                level=NotionDump.DUMP_MODE_DEFAULT)
+            return ""
+        formula_block = block_handle["formula"]
         ret_str = ""
-        if rollup_block[rollup_block["type"]] is None:
-            return ret_str
-        ret_str = rollup_block[rollup_block["type"]]
+        if formula_block["type"] == "string":
+            ret_str = formula_block["string"]
+        elif formula_block["type"] == "number":
+            ret_str = str(formula_block["number"])
+        elif formula_block["type"] == "boolean":
+            if formula_block["boolean"] is True:
+                ret_str = NotionDump.MD_BOOL_TRUE
+            else:
+                ret_str = NotionDump.MD_BOOL_FALSE
+            # ret_str = str(formula_block["boolean"])
+        elif formula_block["type"] == "date":
+            ret_str = self.date_parser(formula_block)
+        else:
+            ret_str = "[unknown_formula_type:" + formula_block["type"] + "]"
         return ret_str
 
     # 数据库 created_time
@@ -461,7 +542,7 @@ class BaseParser:
             common_op.debug_log("created_time type error! parent_id= " + self.base_id + " id= " + block_handle["id"],
                                 level=NotionDump.DUMP_MODE_DEFAULT)
             return ""
-        return block_handle["created_time"]
+        return format_date_or_time(block_handle["created_time"])
 
     # 数据库 last_edited_time
     def last_edited_time_parser(self, block_handle):
@@ -470,7 +551,7 @@ class BaseParser:
                 "last_edited_time type error! parent_id= " + self.base_id + " id= " + block_handle["id"],
                 level=NotionDump.DUMP_MODE_DEFAULT)
             return ""
-        return block_handle["last_edited_time"]
+        return format_date_or_time(block_handle["last_edited_time"])
 
     def created_by_parser(self, block_handle):
         if block_handle["type"] != "created_by":
@@ -501,7 +582,7 @@ class BaseParser:
             common_op.debug_log("paragraph type error! parent_id= " + self.base_id + " id= " + block_handle["id"],
                                 level=NotionDump.DUMP_MODE_DEFAULT)
             return paragraph_ret
-        return self.__text_list_parser(block_handle["paragraph"]["text"], parser_type)
+        return self.__text_list_parser(block_handle["paragraph"]["rich_text"], parser_type)
 
     # Page heading_1
     def heading_1_parser(self, block_handle, parser_type=NotionDump.PARSER_TYPE_PLAIN):
@@ -510,7 +591,7 @@ class BaseParser:
             common_op.debug_log("heading_1 type error! parent_id= " + self.base_id + " id= " + block_handle["id"],
                                 level=NotionDump.DUMP_MODE_DEFAULT)
             return heading_1_ret
-        heading_1_ret = self.__text_list_parser(block_handle["heading_1"]["text"], parser_type)
+        heading_1_ret = self.__text_list_parser(block_handle["heading_1"]["rich_text"], parser_type)
         if parser_type == NotionDump.PARSER_TYPE_MD:
             return "# " + heading_1_ret
         else:
@@ -523,7 +604,7 @@ class BaseParser:
             common_op.debug_log("heading_2 type error! parent_id= " + self.base_id + " id= " + block_handle["id"],
                                 level=NotionDump.DUMP_MODE_DEFAULT)
             return heading_2_ret
-        heading_2_ret = self.__text_list_parser(block_handle["heading_2"]["text"], parser_type)
+        heading_2_ret = self.__text_list_parser(block_handle["heading_2"]["rich_text"], parser_type)
 
         if parser_type == NotionDump.PARSER_TYPE_MD:
             return "## " + heading_2_ret
@@ -537,7 +618,7 @@ class BaseParser:
             common_op.debug_log("heading_3 type error! parent_id= " + self.base_id + " id= " + block_handle["id"],
                                 level=NotionDump.DUMP_MODE_DEFAULT)
             return heading_3_ret
-        heading_3_ret = self.__text_list_parser(block_handle["heading_3"]["text"], parser_type)
+        heading_3_ret = self.__text_list_parser(block_handle["heading_3"]["rich_text"], parser_type)
 
         if parser_type == NotionDump.PARSER_TYPE_MD:
             return "### " + heading_3_ret
@@ -551,7 +632,7 @@ class BaseParser:
             common_op.debug_log("to_do type error! parent_id= " + self.base_id + " id= " + block_handle["id"],
                                 level=NotionDump.DUMP_MODE_DEFAULT)
             return to_do_ret
-        to_do_ret = self.__text_list_parser(block_handle["to_do"]["text"], parser_type)
+        to_do_ret = self.__text_list_parser(block_handle["to_do"]["rich_text"], parser_type)
 
         if parser_type == NotionDump.PARSER_TYPE_MD:
             if block_handle["to_do"]["checked"]:
@@ -569,7 +650,7 @@ class BaseParser:
                 "bulleted_list_item type error! parent_id= " + self.base_id + " id= " + block_handle["id"],
                 level=NotionDump.DUMP_MODE_DEFAULT)
             return bulleted_list_item_ret
-        bulleted_list_item_ret = self.__text_list_parser(block_handle["bulleted_list_item"]["text"], parser_type)
+        bulleted_list_item_ret = self.__text_list_parser(block_handle["bulleted_list_item"]["rich_text"], parser_type)
 
         if parser_type == NotionDump.PARSER_TYPE_MD:
             return "- " + bulleted_list_item_ret
@@ -584,7 +665,7 @@ class BaseParser:
                 "numbered_list_item type error! parent_id= " + self.base_id + " id= " + block_handle["id"],
                 level=NotionDump.DUMP_MODE_DEFAULT)
             return numbered_list_item_ret
-        numbered_list_item_ret = self.__text_list_parser(block_handle["numbered_list_item"]["text"], parser_type)
+        numbered_list_item_ret = self.__text_list_parser(block_handle["numbered_list_item"]["rich_text"], parser_type)
 
         if parser_type == NotionDump.PARSER_TYPE_MD:
             return str(list_index) + ". " + numbered_list_item_ret
@@ -598,7 +679,7 @@ class BaseParser:
             common_op.debug_log("toggle type error! parent_id= " + self.base_id + " id= " + block_handle["id"],
                                 level=NotionDump.DUMP_MODE_DEFAULT)
             return toggle_ret
-        toggle_ret = self.__text_list_parser(block_handle["toggle"]["text"], parser_type)
+        toggle_ret = self.__text_list_parser(block_handle["toggle"]["rich_text"], parser_type)
 
         if parser_type == NotionDump.PARSER_TYPE_MD:
             return "- " + toggle_ret
@@ -622,7 +703,7 @@ class BaseParser:
             common_op.debug_log("callout type error! parent_id= " + self.base_id + " id= " + block_handle["id"],
                                 level=NotionDump.DUMP_MODE_DEFAULT)
             return callout_ret
-        callout_ret = self.__text_list_parser(block_handle["callout"]["text"], parser_type)
+        callout_ret = self.__text_list_parser(block_handle["callout"]["rich_text"], parser_type)
 
         if parser_type == NotionDump.PARSER_TYPE_MD:
             # 这里是否每一行都操作
@@ -637,7 +718,7 @@ class BaseParser:
             common_op.debug_log("code type error! parent_id= " + self.base_id + " id= " + block_handle["id"],
                                 level=NotionDump.DUMP_MODE_DEFAULT)
             return code_ret
-        code_ret = self.__text_list_parser(block_handle["code"]["text"], parser_type)
+        code_ret = self.__text_list_parser(block_handle["code"]["rich_text"], parser_type)
 
         code_type = block_handle["code"]["language"]
         if code_type is None:
@@ -656,7 +737,21 @@ class BaseParser:
             common_op.debug_log("quote type error! parent_id= " + self.base_id + " id= " + block_handle["id"],
                                 level=NotionDump.DUMP_MODE_DEFAULT)
             return quote_ret
-        quote_ret = self.__text_list_parser(block_handle["quote"]["text"], parser_type)
+        quote_ret = self.__text_list_parser(block_handle["quote"]["rich_text"], parser_type)
+        # 最外层颜色
+        if block_handle["quote"]["color"] != "default":
+            # 添加颜色，区分背景色和前景色
+            if NotionDump.S_THEME_TYPE == "markdown":
+                # 使用markdown默认的高亮来渲染所有的颜色类型
+                quote_ret = NotionDump.MD_HIGHLIGHT + quote_ret + NotionDump.MD_HIGHLIGHT
+            else:
+                if block_handle["quote"]["color"].find("_background") != -1:
+                    bg_color = block_handle["quote"]["color"][0:block_handle["quote"]["color"].rfind('_')]
+                    quote_ret = "<span style=\"background-color:" + color_transformer(bg_color,
+                                                                                      background=True) + "\">" + quote_ret + "</span>"
+                else:
+                    quote_ret = "<font color=\"" + color_transformer(block_handle["quote"]["color"],
+                                                                     background=False) + "\">" + quote_ret + "</font>"
 
         if parser_type == NotionDump.PARSER_TYPE_MD:
             # 这里是否每一行都操作
@@ -772,7 +867,7 @@ class BaseParser:
         common_op.add_new_child_page(
             self.child_pages,
             key_id=image_id,
-            link_id=image_url,
+            link_src=image_url,
             page_type="image",
             page_name=image_name
         )
@@ -793,9 +888,9 @@ class BaseParser:
         else:
             return content_format.get_page_format_plain(image_name)
 
-    # Page file(file,pdf)
+    # Page file(file,pdf,video)
     def file_parser(self, block_handle, parser_type=NotionDump.PARSER_TYPE_PLAIN):
-        if block_handle["type"] != "file" and block_handle["type"] != "pdf":
+        if block_handle["type"] != "file" and block_handle["type"] != "pdf" and block_handle["type"] != "video":
             common_op.debug_log("file type error! parent_id= " + self.base_id + " id= " + block_handle["id"],
                                 level=NotionDump.DUMP_MODE_DEFAULT)
             return ""
@@ -809,21 +904,28 @@ class BaseParser:
             if "url" in block_handle[block_type][file_type].keys():
                 file_url = block_handle[block_type][file_type]["url"]
         if file_url == "":
-            common_op.debug_log("unknown block type" + block_handle[block_type]["type"],
+            common_op.debug_log("unknown block type" + block_handle[block_type]["type"] + " with null url",
                                 level=NotionDump.DUMP_MODE_DEFAULT)
+            return ""
+        # 如果caption中没有文件名，尝试从url中分离
         if file_name == "":
-            # 如果文件没有名字使用id作为默认名字
-            file_name = file_id
+            file_url_basic = file_url[0:file_url.rfind('?')]
+            file_name = file_url_basic[file_url_basic.rfind('/')+1:]
+            # url中分离的内容需要转码
+            file_name = unquote(file_name, 'utf-8')
+        if file_name == "":
+            # 如果文件没有名字使用file作为默认名字
+            file_name = "FILE"
         common_op.add_new_child_page(
             self.child_pages,
             key_id=file_id,
-            link_id=file_url,
+            link_src=file_url,
             page_type="file",
             page_name=file_name
         )
 
         common_op.debug_log(
-            "file_parser add page id = " + file_id + "name : " + file_name, level=NotionDump.DUMP_MODE_DEFAULT)
+            "file_parser add page id = " + file_id + " name : " + file_name, level=NotionDump.DUMP_MODE_DEFAULT)
         common_op.debug_log(internal_var.PAGE_DIC)
         common_op.debug_log("#############")
         common_op.debug_log(self.child_pages)
@@ -847,7 +949,7 @@ class BaseParser:
             return bookmark_ret
         bookmark_name = self.__text_list_parser(block_handle["bookmark"]["caption"], parser_type)
         if bookmark_name == "":
-            bookmark_name = block_handle["id"]
+            bookmark_name = "BOOKMARK"
         bookmark_url = block_handle["bookmark"]["url"]
 
         # bookmark 类型要返回一个链接占位符，供后续解析使用
@@ -857,6 +959,25 @@ class BaseParser:
         else:
             return content_format.get_file_format_plain(bookmark_name, bookmark_url)
 
+    # Page embed
+    def embed_parser(self, block_handle, parser_type=NotionDump.PARSER_TYPE_PLAIN):
+        embed_ret = ""
+        if block_handle["type"] != "embed":
+            common_op.debug_log("embed type error! parent_id= " + self.base_id + " id= " + block_handle["id"],
+                                level=NotionDump.DUMP_MODE_DEFAULT)
+            return embed_ret
+        embed_name = self.__text_list_parser(block_handle["embed"]["caption"], parser_type)
+        if embed_name == "":
+            embed_name = "EMBED"
+        embed_url = block_handle["embed"]["url"]
+
+        # bookmark 类型要返回一个链接占位符，供后续解析使用
+        if parser_type == NotionDump.PARSER_TYPE_MD:
+            # file转换成文件链接的形式
+            return content_format.get_file_format_md(embed_name, embed_url)
+        else:
+            return content_format.get_file_format_plain(embed_name, embed_url)
+
     # Page link_preview
     def link_preview_parser(self, block_handle, parser_type=NotionDump.PARSER_TYPE_PLAIN):
         link_preview_ret = ""
