@@ -9,8 +9,8 @@ from NotionDump.Dump.dump import Dump
 from NotionDump.Notion.Notion import NotionQuery
 from NotionDump.utils import common_op
 
-TOKEN_TEST = "secret_g6easunNXs3S5OdTS4qBhlPZN9fjxTkdJ80wOImK5cZ"
-PAGE_MIX_ID = "ed0a3b0f57b34712bc6bafcbdb413d50"
+TOKEN_TEST = "secret_ALjbBRGaZcagEjPtL1c2F139steBXjr8Fc8uQso4YLV"
+PAGE_MIX_ID = "921e6b4ea44046c6935bcb2c69453196"
 # NotionDump.DUMP_MODE = NotionDump.DUMP_MODE_DEBUG
 
 
@@ -33,12 +33,21 @@ def test_page_parser(query, export_child=False, db_parser_type=NotionDump.PARSER
     )
 
 
+def test_page_retrieve(query: NotionQuery):
+    # 获取页面信息测试
+    ret = query.retrieve_page("0cee7c12f04c4157bcc025355adf2312")
+    print(ret)
+
+
 if __name__ == '__main__':
     query_handle = NotionQuery(token=TOKEN_TEST)
     if query_handle is None:
         logging.exception("query handle init error")
         exit(-1)
 
+    # 测试收集页面信息
+    # test_page_retrieve(query_handle)
+
     # 页面解析测试,递归
     test_page_parser(query_handle, True)
 
